@@ -1,3 +1,4 @@
+import collections
 import os
 import six
 import sqlite3
@@ -6,10 +7,62 @@ import warnings
 from gffutils import bins
 from gffutils import helpers
 from gffutils import constants
+from gffutils import merge_criteria as mc
 from gffutils.feature import Feature
 from gffutils.exceptions import FeatureNotFoundError
 
 
+def assign_child(parent, child):
+    """
+    Helper for add_relation()
+    Sets 'Parent' attribute to parent['ID']
+
+    Parameters
+    ----------
+
+    parent : Feature
+        Parent Feature
+
+    :param parent: parent Feature
+
+    child : Feature
+        Child Feature
+
+    Returns
+    -------
+
+    Child Feature
+    """
+    child.attributes['Parent'] = parent['ID']
+    return child
+
+
+# Reusable constant for FeatureDB.merge()
+no_children = tuple()
+
+def _finalize_merge(feature, feature_children):
+    """
+    Helper for FeatureDB.merge() to update source and assign children property
+
+    Parameters
+    ----------
+    feature : Feature
+        feature to finalise
+    
+    feature_children
+        list of children to assign
+    
+    Returns
+    -------
+    feature, modified
+    """
+    if len(feature_children) > 1:
+        feature.source = ','.join(set(child.source for child in feature_children))
+        feature.children = feature_children
+    else:
+        feature.children = no_children
+    return feature
+
 class FeatureDB(object):
     # docstring to be filled in for methods that call out to
     # helpers.make_query()
@@ -147,7 +200,7 @@ class FeatureDB(object):
             '''
             SELECT base, n FROM autoincrements
             ''')
-        self._autoincrements = dict(c)
+        self._autoincrements = collections.defaultdict(int, c)
 
         self.set_pragmas(pragmas)
 
@@ -604,17 +657,15 @@ class FeatureDB(object):
         # Only use bins if we have defined boundaries and completely_within is
         # True. Otherwise you can't know how far away a feature stretches
         # (which means bins are not computable ahead of time)
+        _bin_clause = ''
         if (start is not None) and (end is not None) and completely_within:
-            _bins = list(bins.bins(start, end, one=False))
-            # See issue #45
-            if len(_bins) < 900:
-                _bin_clause = ' or ' .join(['bin = ?' for _ in _bins])
-                _bin_clause = 'AND ( %s )' % _bin_clause
-                args += _bins
-            else:
-                _bin_clause = ''
-        else:
-            _bin_clause = ''
+            if start <= bins.MAX_CHROM_SIZE and end <= bins.MAX_CHROM_SIZE:
+                _bins = list(bins.bins(start, end, one=False))
+                # See issue #45
+                if len(_bins) < 900:
+                    _bin_clause = ' or ' .join(['bin = ?' for _ in _bins])
+                    _bin_clause = 'AND ( %s )' % _bin_clause
+                    args += _bins
 
         query = ' '.join([
             constants._SELECT,
@@ -662,11 +713,9 @@ class FeatureDB(object):
         Providing N features will return N - 1 new features.
 
         This method purposefully does *not* do any merging or sorting of
-        coordinates, so you may want to use :meth:`FeatureDB.merge` first.
-
-        The new features' attributes will be a merge of the neighboring
-        features' attributes.  This is useful if you have provided a list of
-        exons; the introns will then retain the transcript and/or gene parents.
+        coordinates, so you may want to use :meth:`FeatureDB.merge` first, or
+        when selecting features use the `order_by` kwarg, e.g.,
+        `db.features_of_type('gene', order_by=('seqid', 'start'))`.
 
         Parameters
         ----------
@@ -678,6 +727,14 @@ class FeatureDB(object):
             then the featuretypes will be constructed from the neighboring
             features, e.g., `inter_exon_exon`.
 
+        merge_attributes : bool
+            If True, new features' attributes will be a merge of the neighboring
+            features' attributes.  This is useful if you have provided a list of
+            exons; the introns will then retain the transcript and/or gene
+            parents as a single item. Otherwise, if False, the attribute will
+            be a comma-separated list of values, potentially listing the same
+            gene ID twice.
+
         attribute_func : callable or None
             If None, then nothing special is done to the attributes.  If
             callable, then the callable accepts two attribute dictionaries and
@@ -705,9 +762,21 @@ class FeatureDB(object):
             if new_featuretype is None:
                 new_featuretype = 'inter_%s_%s' % (
                     last_feature.featuretype, f.featuretype)
-            assert last_feature.strand == f.strand
-            assert last_feature.chrom == f.chrom
-            strand = last_feature.strand
+            if last_feature.strand != f.strand:
+                new_strand = '.'
+            else:
+                new_strand = f.strand
+
+            if last_feature.chrom != f.chrom:
+                # We've moved to a new chromosome.  For example, if we're
+                # getting intergenic regions from all genes, they will be on
+                # different chromosomes. We still assume sorted features, but
+                # don't complain if they're on different chromosomes -- just
+                # move on.
+                last_feature = f
+                continue
+
+            strand = new_strand
             chrom = last_feature.chrom
 
             # Shrink
@@ -748,6 +817,7 @@ class FeatureDB(object):
                     dialect = None
             yield self._feature_returner(**fields)
             interfeature_start = f.stop
+            last_feature = f
 
     def delete(self, features, make_backup=True, **kwargs):
         """
@@ -797,7 +867,19 @@ class FeatureDB(object):
 
     def update(self, data, make_backup=True, **kwargs):
         """
-        Update database with features in `data`.
+        Update the on-disk database with features in `data`.
+
+        WARNING: If you used any non-default kwargs for gffutils.create_db when
+        creating the database in the first place (especially
+        `disable_infer_transcripts` or `disable_infer_genes`) then you should
+        use those same arguments here.
+
+        The returned object is the same FeatureDB, but since it is pointing to
+        the same database and that has been just updated, the new features can
+        now be accessed.
+
+        Parameters
+        ----------
 
         data : str, iterable, FeatureDB instance
             If FeatureDB, all data will be used. If string, assume it's
@@ -810,15 +892,15 @@ class FeatureDB(object):
             makes a copy of the existing database and saves it with a .bak
             extension.
 
-        Notes
-        -----
-        Other kwargs are used in the same way as in gffutils.create_db; see the
-        help for that function for details.
+        kwargs :
+            Additional kwargs are passed to gffutils.create_db; see the help
+            for that function for details.
 
         Returns
         -------
-        FeatureDB with updated features.
+        Returns self but with the underlying database updated.
         """
+
         from gffutils import create
         from gffutils import iterators
         if make_backup:
@@ -833,6 +915,7 @@ class FeatureDB(object):
 
         # Handle all sorts of input
         data = iterators.DataIterator(data, **_iterator_kwargs)
+        kwargs['_autoincrements'] = self._autoincrements
 
         if self.dialect['fmt'] == 'gtf':
             if 'id_spec' not in kwargs:
@@ -849,10 +932,18 @@ class FeatureDB(object):
         else:
             raise ValueError
 
+        peek, data._iter = iterators.peek(data._iter, 1)
+        if len(peek) == 0: return db  # If the file is empty then do nothing
+
         db._populate_from_lines(data)
         db._update_relations()
+
+        # Note that the autoincrements gets updated here
         db._finalize()
-        return db
+
+        # Read it back in directly from the stored autoincrements table
+        self._autoincrements.update(db._autoincrements)
+        return self
 
     def add_relation(self, parent, child, level, parent_func=None,
                      child_func=None):
@@ -1000,8 +1091,11 @@ class FeatureDB(object):
             ):
                 yield intron
 
-    def merge(self, features, ignore_strand=False):
+    def _old_merge(self, features, ignore_strand=False):
         """
+        DEPRECATED, only retained here for backwards compatibility. Please use
+        merge().
+
         Merge overlapping features together.
 
         Parameters
@@ -1095,6 +1189,197 @@ class FeatureDB(object):
             attributes='')
         yield self._feature_returner(**merged_feature)
 
+
+    def merge(self, features,
+              merge_criteria=(mc.seqid, mc.overlap_end_inclusive, mc.strand, mc.feature_type),
+              multiline=False):
+        """
+        Merge features matching criteria together
+
+
+        Returned Features have a special property called 'children' that is
+        a list of the component features.  This only exists for the lifetime of
+        the Feature instance.
+
+        Parameters
+        ----------
+        features : iterable
+            Iterable of Feature instances to merge
+
+        merge_criteria : list
+            List of merge criteria callbacks. All must evaluate to True in
+            order for a feature to be merged. See notes below on callback
+            signature.
+
+        multiline : bool
+            True to emit multiple features with the same ID attribute, False
+            otherwise.
+
+        Returns
+        -------
+        Generator yielding merged Features
+
+        Notes
+        -----
+
+        See the `gffutils.merge_criteria` module (imported here as `mc`) for
+        existing callback functions. For writing custom callbacks, functions
+        must have the following signature::
+
+            callback(
+                acc: gffutils.Feature,
+                cur: gffutils.Feature,
+                components: [gffutils.Feature]
+            ) -> bool
+
+        Where:
+
+            - `acc`: current accumulated feature
+            - `cur`: candidate feature to merge
+            - `components`: list of features that compose acc
+
+        The function should return True to merge `cur` into `acc`, False to set
+        `cur` to `acc` (that is, start a new merged feature).
+
+
+        If merge criteria allows different feature types then the merged
+        features' feature types should have their featuretype property
+        reassigned to a more specific ontology value.
+        """
+        if not isinstance(merge_criteria, list):
+            try:
+                merge_criteria = list(merge_criteria)
+            except TypeError:
+                merge_criteria = [merge_criteria]
+
+        # To start, we create a merged feature of just the first feature.
+        features = iter(features)
+        last_id = None
+        current_merged = None
+        feature_children = []
+
+        for feature in features:
+            if current_merged is None:
+                if all(criteria(feature, feature, feature_children) for criteria in merge_criteria):
+                    current_merged = feature
+                    feature_children = [feature]
+                else:
+                    yield _finalize_merge(feature, no_children)
+                    last_id = None
+                continue
+
+            if len(feature_children) == 0:  # current_merged is last feature and unchecked
+                if all(criteria(current_merged, current_merged, feature_children) for criteria in merge_criteria):
+                    feature_children.append(current_merged)
+                else:
+                    yield _finalize_merge(current_merged, no_children)
+                    current_merged = feature
+                    last_id = None
+                    continue
+
+            if all(criteria(current_merged, feature, feature_children) for criteria in merge_criteria):
+                # Criteria satisfied, merge
+                # TODO Test multiline records and iron out the following code
+                # if multiline and (feature.start > current_merged.end + 1 or feature.end + 1 < current_merged.start):
+                #    # Feature is possibly multiline (discontiguous), keep ID but start new record
+                #    yield _finalize_merge(current_merged, feature_children)
+                #    current_merged = feature
+                #    feature_children = [feature]
+
+                if len(feature_children) == 1:
+                    # Current merged is only child and merge is going to occur, make copy
+                    current_merged = vars(current_merged).copy()
+                    del current_merged['attributes']
+                    del current_merged['extra']
+                    del current_merged['dialect']
+                    del current_merged['keep_order']
+                    del current_merged['sort_attribute_values']
+                    current_merged = self._feature_returner(**current_merged)
+                    if not last_id:
+                        # Generate unique ID for new Feature
+                        self._autoincrements[current_merged.featuretype] += 1
+                        last_id = current_merged.featuretype + '_' + str(
+                            self._autoincrements[current_merged.featuretype])
+                    current_merged['ID'] = last_id
+                    current_merged.id = last_id
+
+                feature_children.append(feature)
+
+                # Set mismatched properties to ambiguous values
+                if feature.seqid not in current_merged.seqid.split(','): current_merged.seqid += ',' + feature.seqid
+                if feature.strand != current_merged.strand: current_merged.strand = '.'
+                if feature.frame != current_merged.frame: current_merged.frame = '.'
+                if feature.featuretype != current_merged.featuretype: current_merged.featuretype = "sequence_feature"
+
+                if feature.start < current_merged.start:
+                    # Extends prior, so set a new start position
+                    current_merged.start = feature.start
+
+                if feature.end > current_merged.end:
+                    # Extends further, so set a new stop position
+                    current_merged.end = feature.end
+
+            else:
+                yield _finalize_merge(current_merged, feature_children)
+                current_merged = feature
+                feature_children = []
+                last_id = None
+
+        if current_merged:
+            yield _finalize_merge(current_merged, feature_children)
+
+    def merge_all(self,
+                  merge_order=('seqid', 'featuretype', 'strand', 'start'),
+                  merge_criteria=(mc.seqid, mc.overlap_end_inclusive, mc.strand, mc.feature_type),
+                  featuretypes_groups=(None,),
+                  exclude_components=False):
+        """
+        Merge all features in database according to criteria.
+        Merged features will be assigned as children of the merged record.
+        The resulting records are added to the database.
+
+        Parameters
+        ----------
+        merge_order : list
+            Ordered list of columns with which to group features before evaluating criteria
+        merge_criteria : list
+            List of merge criteria callbacks. See merge().
+        featuretypes_groups : list
+            iterable of sets of featuretypes to merge together
+        exclude_components : bool
+            True: child features will be discarded. False to keep them.
+
+        Returns
+        -------
+            list of merge features
+        """
+
+        if not len(featuretypes_groups):
+            # Can't be empty
+            featuretypes_groups = (None,)
+
+        result_features = []
+
+        # Merge features per featuregroup
+        for featuregroup in featuretypes_groups:
+            for merged in self.merge(self.all_features(featuretype=featuregroup, order_by=merge_order),
+                                merge_criteria=merge_criteria):
+                # If feature is result of merge
+                if merged.children:
+                    self._insert(merged, self.conn.cursor())
+                    if exclude_components:
+                        # Remove child features from DB
+                        self.delete(merged.children)
+                    else:
+                        # Add child relations to DB
+                        for child in merged.children:
+                            self.add_relation(merged, child, 1, child_func=assign_child)
+                    result_features.append(merged)
+                else:
+                    pass  # Do nothing, feature is already in DB
+
+        return result_features
+
     def children_bp(self, feature, child_featuretype='exon', merge=False,
                     ignore_strand=False):
         """
