@@ -120,7 +120,7 @@ def test_update():
         # Add attribute to each gene record
         rec = gene_recs[0]
         rec.attributes["new"] = ["new_value"]
-        db.update([rec])
+        db.update([rec], merge_strategy='replace')
         num_entries += 1
     print(list(db.all_features()))
 
@@ -621,6 +621,7 @@ def test_create_db_from_url():
 
     # Serving test/data folder
     served_folder = gffutils.example_filename('')
+    savedir = os.getcwd()
     os.chdir(served_folder)
 
     print("Starting SimpleHTTPServer in thread")
@@ -654,6 +655,7 @@ def test_create_db_from_url():
         print('Server shutdown.')
         httpd.shutdown()
         server_thread.join()
+        os.chdir(savedir)
 
 
 def test_empty_files():
@@ -880,7 +882,7 @@ def test_tempfiles():
     assert len(filelist) == 1, filelist
     assert filelist[0].endswith('.gffutils')
 
-    #...and another one for gff. This time, make sure the suffix 
+    #...and another one for gff. This time, make sure the suffix
     db = gffutils.create_db(
         gffutils.example_filename('FBgn0031208.gff'), ':memory:', _keep_tempfiles=True)
     filelist = os.listdir(tempdir)
@@ -1006,7 +1008,7 @@ def test_disable_infer():
 
 
 def test_deprecation_handler():
-    return 
+    return
 
     # TODO: when infer_gene_extent actually gets deprecated, test here.
     assert_raises(ValueError, gffutils.create_db,
@@ -1141,7 +1143,7 @@ def test_unreasonable_unquoting():
     assert f.attributes['null'][0] == '\x00'
     assert f.attributes['comma'][0] == ','
 
-    # Commas indicate 
+    # Commas indicate
     assert f.attributes['Parent'] == ['A,', 'B%', 'C']
     assert str(f) == s
 
@@ -1174,6 +1176,145 @@ def test_db_unquoting():
     assert db['e']['Note'] == [',']
     assert db['f']['Note'] == [',']
 
+
+def test_issue_105():
+    fn = gffutils.example_filename('FBgn0031208.gtf')
+    home = os.path.expanduser('~')
+    newfn = os.path.join(home, '.gffutils.test')
+    with open(newfn, 'w') as fout:
+        fout.write(open(fn).read())
+    f = gffutils.iterators.DataIterator(newfn)
+    for i in f:
+        pass
+    os.unlink(newfn)
+
+
+def test_issue_107():
+    s = dedent(
+        '''
+        chr1\t.\tgene\t10\t15\t.\t+\t.\tID=b;
+        chr1\t.\tgene\t1\t5\t.\t-\t.\tID=a;
+        chr2\t.\tgene\t25\t50\t.\t-\t.\tID=c;
+        chr2\t.\tgene\t55\t60\t.\t-\t.\tID=d;
+        ''')
+    tmp = tempfile.NamedTemporaryFile(delete=False).name
+    with open(tmp, 'w') as fout:
+        fout.write(s + '\n')
+    db = gffutils.create_db(tmp, ':memory:')
+    interfeatures = list(db.interfeatures(
+        db.features_of_type('gene', order_by=('seqid', 'start'))))
+    assert [str(i) for i in interfeatures] == [
+            'chr1\tgffutils_derived\tinter_gene_gene\t6\t9\t.\t.\t.\tID=a,b;',
+            'chr2\tgffutils_derived\tinter_gene_gene\t16\t54\t.\t-\t.\tID=c,d;',
+    ]
+
+
+def test_issue_119():
+    # First file has these two exons with no ID:
+    #
+    #   chr2L FlyBase exon  8193  8589  .  +  .  Parent=FBtr0300690
+    #   chr2L FlyBase exon  7529  8116  .  +  .  Name=CG11023:1;Parent=FBtr0300689,FBtr0300690
+    #
+    db0  = gffutils.create_db(gffutils.example_filename('FBgn0031208.gff'),':memory:')
+
+    # And this one, a bunch of reads with no IDs anywhere
+    db1 = gffutils.create_db(gffutils.example_filename('F3-unique-3.v2.gff'),':memory:')
+
+    # When db1 is updated by db0
+    db2 = db1.update(db0)
+    assert (
+        db2._autoincrements
+        == db1._autoincrements
+        == {'exon': 2, 'read': 112}
+    ), db2._autoincrements
+
+
+    assert len(list(db0.features_of_type('exon'))) == 6
+
+    # Now we update that with db0 again
+    db3 = db2.update(db0, merge_strategy='replace')
+
+    # Using the "replace" strategy, we should have only gotten another 2 exons
+    assert len(list(db3.features_of_type('exon'))) == 8
+
+    # Make sure that the autoincrements for exons jumped by 2
+    assert (
+        db2._autoincrements
+        == db3._autoincrements
+        == {'exon': 4, 'read': 112}
+    ), db2._autoincrements
+
+    # More isolated test, merging two databases each created from the same file
+    # which itself contains only a single feature with no ID.
+    tmp = tempfile.NamedTemporaryFile(delete=False).name
+    with open(tmp, 'w') as fout:
+        fout.write('chr1\t.\tgene\t10\t15\t.\t+\t.\t\n')
+
+    db4 = gffutils.create_db(tmp, tmp + '.db')
+    db5 = gffutils.create_db(tmp, ':memory:')
+
+    assert db4._autoincrements == {'gene': 1}
+    assert db5._autoincrements == {'gene': 1}
+
+    db6 = db4.update(db5)
+
+    db7 = gffutils.FeatureDB(db4.dbfn)
+
+    # both db4 and db6 should now have the same, updated autoincrements because
+    # they both point to the same db.
+    assert db6._autoincrements == db4._autoincrements == {'gene': 2}
+
+    # But db5 was created independently and should have unchanged autoincrements
+    assert db5._autoincrements == {'gene': 1}
+
+    # db7 was created from the database pointed to by both db4 and db6. This
+    # tests that when a FeatureDB is created it should have the
+    # correctly-updated autoincrements read from the db
+    assert db7._autoincrements == {'gene': 2}
+
+def test_pr_131():
+    db  = gffutils.create_db(gffutils.example_filename('FBgn0031208.gff'),':memory:')
+
+    # previously would raise ValueError("No lines parsed -- was an empty
+    # file provided?"); now just does nothing
+    db2 = db.update([])
+
+
+def test_pr_133():
+    # Previously, merge_attributes would not deep-copy the values from the
+    # second dict, and when the values are then modified, the second dict is
+    # unintentionally modified.
+    d1 = {'a': [1]}
+    d2 = {'a': [2]}
+    d1a = {'a': [1]}
+    d2a = {'a': [2]}
+    d3 = gffutils.helpers.merge_attributes(d1, d2)
+    assert d1 == d1a, d1
+    assert d2 == d2a, d2
+
+
+def test_pr_139():
+    db  = gffutils.create_db(gffutils.example_filename('FBgn0031208.gff'),':memory:')
+    exons = list(db.features_of_type('exon'))
+    inter = list(db.interfeatures(exons))
+
+    # previously, the first exon's attributes would show up in subsequent merged features
+    assert exons[0].attributes['Name'][0] not in inter[1].attributes['Name']
+    assert exons[0].attributes['Name'][0] not in inter[2].attributes['Name']
+    assert exons[0].attributes['Name'][0] not in inter[3].attributes['Name']
+
+
+def test_pr_144():
+    # previously this would fail with:
+    #   UnboundLocalError: local variable 'part' referenced before assignment
+    f = gffutils.Feature(attributes={'a': ['']})
+
+    # Make sure everything got converted correctly
+    assert f.attributes['a'] == ['']
+    assert str(f) == ".	.	.	.	.	.	.	.	a"
+    g = gffutils.feature.feature_from_line(str(f))
+    assert g == f
+
 if __name__ == "__main__":
     # this test case fails
     #test_attributes_modify()
@@ -1181,3 +1322,6 @@ if __name__ == "__main__":
     #test_random_chr()
     #test_nonascii()
     test_iterator_update()
+
+
+
